/*
 * GestoreRubrica.java
 *
 * Created on 20 settembre 2003, 12.57
 * @author: Giulio Destri
 */

package rubricaLib;

import java.util.*;
import java.sql.*;
import java.io.*;
import entityLib.Persona;
import exceptionLib.RubricaException;

/**
 *
 * @author  Giulio Destri
 */
public class GestoreDBRubrica {
    
    static Connection dbcon;

    private final static String DB_DRIVERNAME = "sun.jdbc.odbc.JdbcOdbcDriver";
    private final static String DB_URL = "jdbc:odbc:";
    private final static String DB_DS_NAME = "rubrica"; 
    private final static String DB_USERNAME = "";
    private final static String DB_PASSWORD = "";
    
    /** 
     * Creates a new instance of GestoreRubrica 
     * Costruttore, richiama il metodo di connessione al DB
     */
    public GestoreDBRubrica() {
        connectToDB();
        //caricaRubricaDB();
    }
    
    /**
     * connectToDB, si connette al DB usando i 5 parametri definiti come 
     * attributi della classe
     * Creation date: (4/11/2002 11.46.52)
     * 
     */
    public void connectToDB() {

        try {
            // determinazione del driver
	    Class.forName(DB_DRIVERNAME);

            // creazione della connessione
	    dbcon = DriverManager.getConnection(
		DB_URL+DB_DS_NAME, DB_USERNAME, DB_PASSWORD);
            
 	} catch (ClassNotFoundException cnfe) { // class exception: non si trova il driver
	      System.err.println("Errore Fatale --- non possibile caricare il driver "+DB_DRIVERNAME);
	      System.exit(-1); // il programma termina
	} catch (SQLException sqle) { // SQL exception: username o passwd errati
	      System.err.println("Errore Fatale --- non possibile connettere l'utente:"+DB_USERNAME+" al DB "+DB_URL+DB_DS_NAME);
	      //System.exit(-1);
	}

	try {
            // determina informazioni sul database
            DatabaseMetaData dbmd= dbcon.getMetaData();

            // stampa le informazioni su stdout
            System.out.println(
                "Connesso a:"+ dbmd.getDatabaseProductName()+
                    	" "+ dbmd.getDatabaseProductVersion()+
		   " ["+DB_DS_NAME+"]\n");

	} catch (SQLException sqle) { // SQL exception: non si hanno info sul DB
	   System.err.println("Errore fatale (2) --- non posso ottenere info sul database");
	   //System.exit(-2);
	} 
	
	System.gc();	// clean up della memoria

    } // end ConnectToDB
    
    
    
    public void inserisciNuovoDB(Persona p_persona) {
        
        String totalQuery = "INSERT into elenco (nome, cognome, numtel) values (?,?,?) ";
        
	try {
            // crea lo statement
            PreparedStatement pstmt = dbcon.prepareStatement(totalQuery);
                
            pstmt.setString(1,p_persona.getNome());
            pstmt.setString(2,p_persona.getCognome());
            pstmt.setString(3,p_persona.getNumtel());

            // applica l'insert
            pstmt.executeUpdate ();
            System.out.println("Inserimento Esegeguito Correttamente");
        
        } catch (SQLException sqle) { // SQL exception
            System.err.println("Errore Fatale --- non possibile inserire:"+DB_URL+DB_DS_NAME);
            sqle.printStackTrace();
        } // end catch       
           
    } // end inseriscinuovoDB


    public void cancellaDB(Persona p_persona) {
        
        String totalQuery = "DELETE from elenco WHERE (cognome=?) and (nome=?) and (numtel=?) ";
        
	try {
            // crea lo statement
            PreparedStatement pstmt = dbcon.prepareStatement(totalQuery);
               
            // inserisce gli elementi per la ricerca del dato da rimuovere
            pstmt.setString(1,p_persona.getCognome());
            pstmt.setString(2,p_persona.getNome());
            pstmt.setString(3,p_persona.getNumtel());
                
            // applica il delete
            pstmt.executeUpdate ();
            System.out.println("Cancellazione Esegeguita Correttamente");
        
        } catch (SQLException sqle) { // SQL exception
            System.err.println("Errore Fatale --- non possibile Cancellare il dato in:"+DB_URL+DB_DS_NAME);
        } // end catch        
        
    } // end inseriscinuovoDB
    
    
    public void rimpiazzaDB(Persona p_persona_valore, Persona p_persona_chiave) {
        
        String totalQuery = "UPDATE elenco set Nome=?,Cognome=?,Numtel=? WHERE (nome=?) and (cognome=?) and (numtel=?)";
            
        try {
            // crea lo statement
            PreparedStatement pstmt = dbcon.prepareStatement(totalQuery);
                
            // imposta nell'istruzione i valori da inserire
            pstmt.setString(1,p_persona_valore.getNome());
            pstmt.setString(2,p_persona_valore.getCognome());
            pstmt.setString(3,p_persona_valore.getNumtel());
                
            // inserisce i parametri per la ricerca dell'elemento da sostituire
            pstmt.setString(4,p_persona_chiave.getNome());
            pstmt.setString(5,p_persona_chiave.getCognome());
            pstmt.setString(6,p_persona_chiave.getNumtel());
                
            // applica la query ed ottiene il resultset
            pstmt.executeUpdate ();
            System.out.println("Rimpiazzamento Esegeguito Correttamente");
        
       } catch (SQLException sqle) { // SQL exception
            System.err.println("Errore Fatale --- non possibile aggiornamento:"+DB_URL+DB_DS_NAME);
       } // end catch
       
    } // end method rimpiazzaDB    
    
  
    
    public Vector caricaRubricaDB() throws RubricaException {
        Persona temporaneo = null;
        Vector listaPersone = new Vector();
        String totalQuery = "SELECT Nome, Cognome, Numtel from elenco";
        ResultSet rs;
	int numCols;
	int numRows;
	int row = 1;

	try {
		// crea lo statement
		PreparedStatement pstmt = dbcon.prepareStatement(totalQuery);
                
                // inserisce l'argomento, con i wildcard del DB agli estremi 
                // (ricerca per sottostringa)
		//pstmt.setString(1,"");

		// applica la query ed ottiene il resultset
		rs = pstmt.executeQuery ();
                
                
		// conta le colonne presenti
		numCols = rs.getMetaData().getColumnCount();

		// stampa i nomi delle colonne presenti su stdout
		for (int col=1; col<=numCols; col++) {
                    System.out.print(rs.getMetaData().getColumnName(col) + " | "); 	
		}
		System.out.println("|");
                
                                
                //ricerca per tutto il resultset
		// legge tutte le righe del ResultSet
		while (rs.next()) { 	
                    // costruisce l'oggetto Persona contenente i dati dell'ultimo record trovato
                    temporaneo = new Persona(rs.getString(1),rs.getString(2),rs.getString(3));
                    /*
                    for (int col=1; col<=numCols; col++) { 	
			  String string = rs.getString (col);
	 	 	  System.out.print(string + " | "); 
			}
	 		System.out.println("");
	 		row ++;
                     */
                    listaPersone.add(temporaneo);
		} // end while
                 
		
	} catch (SQLException sqle) { // SQL exception
	      System.err.println("Errore Fatale --- non possibile interrogare il DB:"+DB_URL+DB_DS_NAME);
	      // System.exit(-3);
	}
        
        // se il risultato e' ancora null
	if (temporaneo==null){
            // viene lanciata un'eccezione
            System.err.println("Errore");
            RubricaException eccezione;
            eccezione= new RubricaException("Non ho trovato nulla!",RubricaException.NOT_FOUND);
            throw eccezione;
	} // end if (res==null)
        
        return listaPersone;
    } // end caricaRubricaDB
    
    public Vector cercaPerNomeDB(String p_nome) throws RubricaException {
        Persona temporaneo = null;
        Vector listaPersone = new Vector();
        String totalQuery = "SELECT Nome, Cognome, Numtel from elenco where Nome = ?";
        ResultSet rs;
	int numCols;
	int numRows;
	int row = 1;

	try {
		// crea lo statement
		PreparedStatement pstmt = dbcon.prepareStatement(totalQuery);
                
                // inserisce l'argomento, con i wildcard del DB agli estremi 
                // (ricerca per sottostringa)
		pstmt.setString(1,p_nome);

		// applica la query ed ottiene il resultset
		rs = pstmt.executeQuery ();
                
                
		// conta le colonne presenti
		numCols = rs.getMetaData().getColumnCount();

		// stampa i nomi delle colonne presenti su stdout
		for (int col=1; col<=numCols; col++) {
                    System.out.print(rs.getMetaData().getColumnName(col) + " | "); 	
		}
		System.out.println("|");
                
                                
                //ricerca per tutto il resultset
		// legge tutte le righe del ResultSet
		while (rs.next()) { 	
                    // costruisce l'oggetto Persona contenente i dati dell'ultimo record trovato
                    temporaneo = new Persona(rs.getString(1),rs.getString(2),rs.getString(3));
                    /*
                    for (int col=1; col<=numCols; col++) { 	
			  String string = rs.getString (col);
	 	 	  System.out.print(string + " | "); 
			}
	 		System.out.println("");
	 		row ++;
                     */
                    listaPersone.add(temporaneo);
		} // end while
                 
		
	} catch (SQLException sqle) { // SQL exception
	      System.err.println("Errore Fatale --- non possibile interrogare il DB:"+DB_URL+DB_DS_NAME);
	      // System.exit(-3);
	}
        
        // se il risultato e' ancora null
	if (temporaneo==null){
            // viene lanciata un'eccezione
            System.err.println("Errore");
            RubricaException eccezione;
            eccezione= new RubricaException("Non ho trovato nulla!",RubricaException.NOT_FOUND);
            throw eccezione;
	} // end if (res==null)
        
        return listaPersone;
    } // end cercaPerNomeDB
    
    public Vector cercaPerCognomeDB(String p_cognome) throws RubricaException {
        Persona temporaneo = null;
        Vector listaPersone = new Vector();
        String totalQuery = "SELECT Nome, Cognome, Numtel from elenco where Cognome = ?";
        ResultSet rs;
	int numCols;
	int numRows;
	int row = 1;

	try {
		// crea lo statement
		PreparedStatement pstmt = dbcon.prepareStatement(totalQuery);
                
                // inserisce l'argomento, con i wildcard del DB agli estremi 
                // (ricerca per sottostringa)
		pstmt.setString(1,p_cognome);

		// applica la query ed ottiene il resultset
		rs = pstmt.executeQuery ();
                
                
		// conta le colonne presenti
		numCols = rs.getMetaData().getColumnCount();

		// stampa i nomi delle colonne presenti su stdout
		for (int col=1; col<=numCols; col++) {
                    System.out.print(rs.getMetaData().getColumnName(col) + " | "); 	
		}
		System.out.println("|");
                
                                
                //ricerca per tutto il resultset
		// legge tutte le righe del ResultSet
		while (rs.next()) { 	
                    // costruisce l'oggetto Persona contenente i dati dell'ultimo record trovato
                    temporaneo = new Persona(rs.getString(1),rs.getString(2),rs.getString(3));
                    /*
                    for (int col=1; col<=numCols; col++) { 	
			  String string = rs.getString (col);
	 	 	  System.out.print(string + " | "); 
			}
	 		System.out.println("");
	 		row ++;
                     */
                    listaPersone.add(temporaneo);
		} // end while
                 
		
	} catch (SQLException sqle) { // SQL exception
	      System.err.println("Errore Fatale --- non possibile interrogare il DB:"+DB_URL+DB_DS_NAME);
	      // System.exit(-3);
	}
        
        // se il risultato e' ancora null
	if (temporaneo==null){
            // viene lanciata un'eccezione
            System.err.println("Errore");
            RubricaException eccezione;
            eccezione= new RubricaException("Non ho trovato nulla!",RubricaException.NOT_FOUND);
            throw eccezione;
	} // end if (res==null)
        
        return listaPersone;
    } // end cercaPerCognomeDB
    
    public Vector cercaPerNumtelDB(String p_numtel) throws RubricaException {
        Persona temporaneo = null;
        Vector listaPersone = new Vector();
        String totalQuery = "SELECT Nome, Cognome, Numtel from elenco where Numtel = ?";
        ResultSet rs;
	int numCols;
	int numRows;
	int row = 1;

	try {
		// crea lo statement
		PreparedStatement pstmt = dbcon.prepareStatement(totalQuery);
                
                // inserisce l'argomento, con i wildcard del DB agli estremi 
                // (ricerca per sottostringa)
		pstmt.setString(1,p_numtel);

		// applica la query ed ottiene il resultset
		rs = pstmt.executeQuery ();
                
                
		// conta le colonne presenti
		numCols = rs.getMetaData().getColumnCount();

		// stampa i nomi delle colonne presenti su stdout
		for (int col=1; col<=numCols; col++) {
                    System.out.print(rs.getMetaData().getColumnName(col) + " | "); 	
		}
		System.out.println("|");
                
                                
                //ricerca per tutto il resultset
		// legge tutte le righe del ResultSet
		while (rs.next()) { 	
                    // costruisce l'oggetto Persona contenente i dati dell'ultimo record trovato
                    temporaneo = new Persona(rs.getString(1),rs.getString(2),rs.getString(3));
                    /*
                    for (int col=1; col<=numCols; col++) { 	
			  String string = rs.getString (col);
	 	 	  System.out.print(string + " | "); 
			}
	 		System.out.println("");
	 		row ++;
                     */
                    listaPersone.add(temporaneo);
		} // end while
                 
		
	} catch (SQLException sqle) { // SQL exception
	      System.err.println("Errore Fatale --- non possibile interrogare il DB:"+DB_URL+DB_DS_NAME);
	      // System.exit(-3);
	}
        
        // se il risultato e' ancora null
	if (temporaneo==null){
            // viene lanciata un'eccezione
            System.err.println("Errore");
            RubricaException eccezione;
            eccezione= new RubricaException("Non ho trovato nulla!",RubricaException.NOT_FOUND);
            throw eccezione;
	} // end if (res==null)
        
        return listaPersone;
    } // end cercaPerNumtelDB    
    
          
} // end class GestoreDBRubrica
 