// dichiarazione di appartnenenza al package
package rubricaLib;

import java.util.*;
import java.io.*;

import entityLib.Persona;
import exceptionLib.RubricaException;


/**
 * GestoreRubricaHash: classe di servizio che svolge i compiti della rubrica
 * usando internamente le Hashtable
 * Creation date: (25/10/2002 10.37.18)
 * @author: Giulio Destri
 */

public class GestoreRubricaHash {
    /**
     * Attributi interni della classe: listaPersone con i tre possibili 
     * attributi come chiavi
     * Creation date: (25/10/2002 12.39.29)
     */	
    private java.util.Hashtable listaPersonePerCognome;
    private java.util.Hashtable listaPersonePerNome;
    private java.util.Hashtable listaPersonePerNumtel;
    
    /**
    * Costruttore, istanzia le hashtable ma le lascia ancora vuote
    */
    public GestoreRubricaHash() {
	super();
	listaPersonePerCognome=new Hashtable();
        listaPersonePerNome=new Hashtable();
        listaPersonePerNumtel=new Hashtable();
        
        // per immettere dati entro le hashtable
        this.popolaLista();
    } // end costruttore
    
    /**
     * cercaPerCognome: restituisce la Persona in base al cognome
     * Creation date: (25/10/2002 10.46.52)
     * @return RubricaPkg.Persona
     * @param p_cognome java.lang.String 
     */
    public Persona cercaPerCognome(String p_cognome) throws RubricaException {
        Persona res = null;
	res=(Persona)listaPersonePerCognome.get(p_cognome);
        
        // se il risultato e' ancora null
	if (res==null){
            // viene lanciata un'eccezione
            RubricaException errore=new RubricaException();
            errore.setMessaggio("Nessun elemento trovato");
            throw errore;
	} // end if (res==null)
        
        // ritorna il risultato
        return res;
       
    } // end cercaPerCognome

    /**
     * cercaPerNome: restituisce la Persona in base al nome
     * Creation date: (25/10/2002 10.46.52)
     * @return RubricaPkg.Persona
     * @param p_cognome java.lang.String 
     */
    public Persona cercaPerNome(String p_nome) throws RubricaException {
        Persona res = null;
	res=(Persona)listaPersonePerNome.get(p_nome);
        
        // se il risultato e' ancora null
	if (res==null){
            // viene lanciata un'eccezione
            RubricaException errore=new RubricaException();
            errore.setMessaggio("Nessun elemento trovato");
            throw errore;
	} // end if (res==null)
        
        // ritorna il risultato
        return res;
        
    } // end cercaPerNome

    /**
     * cercaPerNumtel: restituisce la Persona in base al numtel
     * Creation date: (25/10/2002 10.46.52)
     * @return RubricaPkg.Persona
     * @param p_cognome java.lang.String 
     */
    public Persona cercaPerNumtel(String p_numtel) throws RubricaException {
        Persona res = null;
	res=(Persona)listaPersonePerNumtel.get(p_numtel);
        
        // se il risultato e' ancora null
	if (res==null){
            // viene lanciata un'eccezione
            RubricaException errore=new RubricaException();
            errore.setMessaggio("Nessun elemento trovato");
            throw errore;
	} // end if (res==null)
        
        // ritorna il risultato
        return res;
        
    } // end cercaPerNumtel
    
    
    /**
     * popolaLista: riempie la lista con nomi fissi
     * Creation date: (25/10/2002 11.00.44)
     */
    public void popolaLista() {
	Persona temporaneo;

        // crea un oggetto persona con i dati
	temporaneo=new Persona("Simone","Mainini","111111");
        // e lo inserisce nelle 3 hashtable con le chiavi opportune
	listaPersonePerCognome.put(temporaneo.getCognome(),(Object)temporaneo);
        listaPersonePerNome.put(temporaneo.getNome(),(Object)temporaneo);
        listaPersonePerNumtel.put(temporaneo.getNumtel(),(Object)temporaneo);

	temporaneo=new Persona("Giulio","Destri","222222");
        listaPersonePerCognome.put(temporaneo.getCognome(),(Object)temporaneo);
        listaPersonePerNome.put(temporaneo.getNome(),(Object)temporaneo);
        listaPersonePerNumtel.put(temporaneo.getNumtel(),(Object)temporaneo);

	temporaneo=new Persona("Gianluca","Golinelli","333333");
	listaPersonePerCognome.put(temporaneo.getCognome(),(Object)temporaneo);
        listaPersonePerNome.put(temporaneo.getNome(),(Object)temporaneo);
        listaPersonePerNumtel.put(temporaneo.getNumtel(),(Object)temporaneo);

	temporaneo=new Persona("Marco","Solci","444444");
	listaPersonePerCognome.put(temporaneo.getCognome(),(Object)temporaneo);
        listaPersonePerNome.put(temporaneo.getNome(),(Object)temporaneo);
        listaPersonePerNumtel.put(temporaneo.getNumtel(),(Object)temporaneo);

	temporaneo=new Persona("Oscar","Figus","5555555");
	listaPersonePerCognome.put(temporaneo.getCognome(),(Object)temporaneo);
        listaPersonePerNome.put(temporaneo.getNome(),(Object)temporaneo);
        listaPersonePerNumtel.put(temporaneo.getNumtel(),(Object)temporaneo);
	
    } // end popolaLista
    

    /**
     * caricaRubrica: legge la rubrica da un file
     * Creation date: (30/10/2002 16.40.22)
     * @param p_filename java.lang.String
     */
    public void caricaRubrica(String p_filename) {
        Persona temporaneo = null;
	String temp_string;
	String separatore = ",";
        
        BufferedReader inputFile; 
        String inputLine;
        StringTokenizer tokenizer;
        
        String f_cognome = "";
        String f_nome = "";
        String f_numtel = "";
        
        try {
            // apre il file in lettura
            inputFile = new BufferedReader(new FileReader(p_filename));
        
            // legge il file riga per riga
            inputLine = inputFile.readLine();
            while (inputLine != null) {
                System.out.println(inputLine);
                // imposta il tokenizzatore per inputLine, con la stringa separatore a fare da separatore
                tokenizer = new StringTokenizer(inputLine,separatore);
                
                // se gli elementi della riga non sono 3
                if (tokenizer.countTokens() != 3) {
                    System.out.println(
                        "numero di token"+
                        tokenizer.countTokens()+
                        "ERRORE nella riga");
                } else  {
                    f_cognome = tokenizer.nextToken();
                    f_nome = tokenizer.nextToken();
                    f_numtel = tokenizer.nextToken();
                                 
                    // crea l'oggetto temporaneo Persona 
                    temporaneo = new Persona(f_nome, f_cognome, f_numtel);
                    // e lo inserisce nelle 3 hashtable con le chiavi opportune
                    listaPersonePerCognome.put(temporaneo.getCognome(),(Object)temporaneo);
                    listaPersonePerNome.put(temporaneo.getNome(),(Object)temporaneo);
                    listaPersonePerNumtel.put(temporaneo.getNumtel(),(Object)temporaneo);
                }
                  
                inputLine = inputFile.readLine();
            } // end while

            // chiude il file 
            inputFile.close();
       
    
        // gestione delle varie eccezioni possibili con i file
	} catch(FileNotFoundException e) {
		System.err.println("File Not Found!");
	} catch(IOException e) {
		System.err.println("I/Oerr");
	} catch(IllegalArgumentException e) {
		System.err.println ("java FileCopy srcdest");
	} finally { /*
		try {
                    //if (inputFile != null) inputFile.close();
		} catch(IOException e) {
                    System.err.println("I/O err");
		} */
	}
    } // end caricaRubrica

    
    
    /**
     * salvaRubrica: salva la rubrica su file
     * Creation date: (30/10/2002 16.40.22)
     * @param p_filename java.lang.String
     */
    public void salvaRubrica(String p_filename) {
	Persona temporaneo = null;
	String temp_string;
	FileOutputStream fout= null;
        String separatore = ",";

	try {
            
            // apre il file in scrittura
            fout = new FileOutputStream(p_filename);            
                      
            // per ciascuna persona (si usa una delle tre liste)                          
            for (Enumeration e = listaPersonePerCognome.elements(); e.hasMoreElements() ;) {
                 
                // estrae un elemento temporaneo
                temporaneo = (Persona)e.nextElement();
                
                // lo converte in stringa estraendone gli attributi
                temp_string=temporaneo.getCognome() +
                    separatore +
                    temporaneo.getNome() +
                    separatore +
                    temporaneo.getNumtel();
                
                // trasforma la stringa in un array di byte (compatibile ASCII)
                byte b[]=temp_string.getBytes();
                
                // scrive l'array nel file
	        fout.write(b);
                
                // definisce il fineriga e lo aggiunge al file
	        byte finRiga[] = "\r\n".getBytes();
	        fout.write(finRiga);
                
            } // end for 
        
            // chiude il file
            fout.close();
            
        // gestione delle varie eccezioni possibili con i file
	} catch(FileNotFoundException e) {
		System.err.println("File Not Found!");
	} catch(IOException e) {
		System.err.println("I/Oerr");
	} catch(IllegalArgumentException e) {
		System.err.println ("java FileCopy srcdest");
	} finally {
		try {
			if (fout != null) fout.close();
		} catch(IOException e) {
			System.err.println("I/O err");
		}
	}
    } // end salvaRubrica


} // end class GestoreRubricaHash
